use anyhow::Result;
use std::env;
use tracing::{info};
use tracing_subscriber::{fmt, EnvFilter};

// Import our library
use vault_hier::{start_api, initialize_vault};

#[tokio::main]
async fn main() -> Result<()> {
    // Initialize tracing
    fmt()
        .with_env_filter(EnvFilter::from_default_env().add_directive("vault_hier=info".parse()?))
        .with_target(false)
        .init();

    // Get Vault address from env var or use default
    let vault_addr = env::var("VAULT_ADDR").unwrap_or_else(|_| "http://127.0.0.1:8200".to_string());

    // Get API port from env var or use default
    let api_port = env::var("API_PORT")
        .unwrap_or_else(|_| "3000".to_string())
        .parse::<u16>()
        .unwrap_or(3000);

    info!("Vault address: {}", vault_addr);
    info!("Connecting to Vault at: {}", vault_addr);

    // Initialize and unseal Vault, get the root token
    let root_token = initialize_vault(&vault_addr).await?;

    info!("Starting hierarchical document signing API...");

    // Start the hierarchical signing API
    start_api(&vault_addr, &root_token, api_port).await?;

    info!("API server shutdown. Exiting.");

    Ok(())
}
